package il.ac.tau.cs.sw1.sortedset.example;

import il.ac.tau.cs.sw1.sortedset.Iterator;
import il.ac.tau.cs.sw1.sortedset.SortedSet;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

/**
 * A simple immutable implementation of the SortedSet interface. This class is
 * provided as an example and for testing purposes.
 */
public class SimpleSortedSet<T extends Comparable<T>> implements SortedSet<T> {
	// Use a  List to implement our own immutable sorted set
	private List<T> values;

	/**
	 * Construct a new SimpleSortedSet with the given array of elements
	 */
	public SimpleSortedSet(T... values) {
		this.values = new ArrayList<T>();
		for (T element : values) {
			if (element == null) { // prohibit the use of null elements
				throw new IllegalArgumentException("null element");
			}
			if (!this.values.contains(element)) {
				this.values.add(element);
			}
		}
		Collections.sort(this.values);
	}

	@Override
	public boolean contains(T element) {
		return values.contains(element);
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (!(obj instanceof SimpleSortedSet<?>))
			return false;
		SimpleSortedSet<?> other = (SimpleSortedSet<?>) obj;
		if (values == null) {
			if (other.values != null)
				return false;
		} else if (!values.equals(other.values))
			return false;
		return true;
	}

	@Override
	public T getMaximum() {
		return values.get(values.size() - 1);
	}

	@Override
	public T getMinimum() {
		return values.get(0);
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((values == null) ? 0 : values.hashCode());
		return result;
	}

	@Override
	public boolean isEmpty() {
		return values.isEmpty();
	}

	@Override
	public Iterator<T> iterator() {
		// implementation of the iterartor as an anonymous class
		return new Iterator<T>() {
			private int index = 0;

			@Override
			public void advance() {
				index++;
			}

			@Override
			public boolean atEnd() {
				return index >= values.size();
			}

			@Override
			public T get() {
				return values.get(index);
			}
		};
	}

	@Override
	public int size() {
		return values.size();
	}

	@Override
	public String toString() {
		return values.toString();
	}

}
